library(reshape)
library(dplyr)

# Produces a frame with the number of final submissions per weekday
# (from the Monday that the lab was released, to the Monday used as
# absolute deadline)
finalSubmissionsByWeekday <- function(path) {
  data <- read.csv(path)
  
  # Fetches the last submission from each author
  finalSubmissions <- data %>% group_by(Author.Email) %>% top_n(1, Attempt)
  
  # Counts how many final submissions we had per calendar day, and adds weekday
  finalSubmissionsByDay <- (finalSubmissions
                            %>% mutate(day = as.Date(Created.at))
                            %>% group_by(Assessment.Title)
                            %>% count(day)
                            %>% mutate(weekday = weekdays(day)))
  
  mondayToMonday(finalSubmissionsByDay) %>% mutate(path = path)
}

# Limits rows to go from the first Monday to the second Monday (both included)
#
# Needs a 'weekday' column
mondayToMonday <- function(data) {
  marks <- (data
     %>% filter(cumsum(weekday == "Monday") > 0)
     %>% group_by(mondays = cumsum(weekday == "Monday"), .add = TRUE)
     %>% mutate(grouprow = row_number() - 1)
     %>% group_by(day)
     %>% mutate(weekday = ifelse(mondays < 2 || mondays == 2 && weekday == "Monday", paste0(weekday, mondays), "Later"))
     %>% select(Assessment.Title, day, n, weekday))

  levels(marks$Assessment.Title)[levels(marks$Assessment.Title) == 'Day::makeAppointment'] <- 'Lab 1: TDD'
  levels(marks$Assessment.Title)[levels(marks$Assessment.Title) == 'Inheritance lab'] <- 'Lab 2: inheritance'
  levels(marks$Assessment.Title)[levels(marks$Assessment.Title) == 'Abstract classes and interfaces'] <- 'Lab 3: interfaces 1'
  levels(marks$Assessment.Title)[levels(marks$Assessment.Title) == 'Lab 4: Foxes and Rabbits from CRC cards'] <- 'Lab 4: interfaces 2'
  levels(marks$Assessment.Title)[levels(marks$Assessment.Title) == 'Lab 7: Java Collections Framework and generics'] <- 'Lab 7: collections'
  levels(marks$Assessment.Title)[levels(marks$Assessment.Title) == 'Lab 8: JavaFX preparatory lab'] <- 'Lab 8: JavaFX prep'
  levels(marks$Assessment.Title)[levels(marks$Assessment.Title) == 'Lab 9: JavaFX assessed lab'] <- 'Lab 9: JavaFX assessed'
  levels(marks$Assessment.Title)[levels(marks$Assessment.Title) == 'Lab 10: File I/O and Exceptions'] <- 'Lab 10: I/O and exceptions'

  marks
}

# Prints a table in LaTeX format to a path
printTable <- function(data, path) {
  print(xtable(data), include.rownames = FALSE, booktabs = TRUE, file=path)
}

# Calls a function creating tables for each file in a folder, and combines them into one long table
applyFolder <- function(folder, func) {
  files <- grep('lab[0-9]+([-]final)?[-]submissions([-]anonymised)?[.]csv$', list.files(folder), value=TRUE)
  byWeek <- lapply(paste(folder, files, sep='/'), func)
  bind_rows(byWeek)
}

# Turns a list of long tables (with n/weekday columns) into a wide table (from Monday to Monday) and row totals
wideMondayToMonday <- function(byWeek) {
  yearRows <- bind_rows(byWeek)
  aggByWeekday <- aggregate(n ~ Assessment.Title + weekday, yearRows, sum)
  wideRows <- cast(aggByWeekday, Assessment.Title ~ weekday, value="n")
  (wideRows[, c('Assessment.Title', 'Monday1', 'Tuesday1', 'Wednesday1', 'Thursday1', 'Friday1', 'Saturday1', 'Sunday1', 'Monday2', 'Later')]
    %>% mutate(Total = as.integer(rowSums(.[2:9], na.rm=TRUE))))
}